# -*- coding: utf-8 -*-
# @Time   : 2020-09-29 4:52 PM
# @Author : Zhiyi Lu
# @Email  : zhiyilv@outlook.com
# @File   : __init__.py.py
# @Desc   : The techtable package's init file


from .funcs import func_dict
from .tt import TechTable

# ----keep at the end
